from datetime import datetime
import os
import csv
import tkinter as tk
from tkinter import ttk


class LabelInput(tk.Frame):
    """A widget containing a label and input together."""

    def __init__(self, parent, label='', input_class=ttk.Entry,
                 input_var=None, input_args=None, label_args=None,
                 **kwargs):
        super().__init__(parent, **kwargs)
        input_args = input_args or {}
        label_args = label_args or {}
        self.variable = input_var

        if input_class in (ttk.Checkbutton, ttk.Button, ttk.Radiobutton):
            input_args["text"] = label
            input_args["variable"] = input_var
        else:
            self.label = ttk.Label(self, text=label, **label_args)
            self.label.grid(row=0, column=0, sticky=(tk.W + tk.E))
            input_args["textvariable"] = input_var

        self.input = input_class(self, **input_args)
        self.input.grid(row=1, column=0)
        self.columnconfigure(0, weight=1)
        # self.columnconfigure(1, weight=2)

    def grid(self, sticky=(tk.E + tk.W), **kwargs):
        super().grid(sticky=sticky, **kwargs)
    # The input widgets all have a get() method that returns their current value.
    def get(self):
        if self.variable:
            return self.variable.get()
        elif type(self.input) == tk.Text:
            return self.input.get('1.0', tk.END)
        else:
            return self.input.get()

    def set(self, value, *args, **kwargs):
        if type(self.variable) == tk.BooleanVar:
                self.variable.set(bool(value))
        elif self.variable:
                self.variable.set(value, *args, **kwargs)
        elif type(self.input).__name__.endswith('button'):
            if value:
                self.input.select()
            else:
                self.input.deselect()
        elif type(self.input) == tk.Text:
            self.input.delete('1.0', tk.END)
            self.input.insert('1.0', value)
        else:
            self.input.delete(0, tk.END)
            self.input.insert(0, value)


class DataRecordForm(tk.Frame):
    """The input form for our widgets"""

    def __init__(self, parent, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        # A dict to keep track of input widgets
        self.inputs = {}

        # Build the form
        # recordinfo section
        recordinfo = tk.LabelFrame(self, text="Record Information")

        # line 1
        self.inputs['Date'] = LabelInput(
            recordinfo, "Date",
            input_var=tk.StringVar()
        )
        self.inputs['Date'].grid(row=0, column=0)
        self.inputs['Time'] = LabelInput(
            recordinfo, "Time",
            input_class=ttk.Combobox,
            input_var=tk.StringVar(),
            input_args={"values": ["8:00", "12:00", "16:00", "20:00"]}
        )
        self.inputs['Time'].grid(row=0, column=1)
        self.inputs['Technician'] = LabelInput(
            recordinfo, "Technician",
            input_var=tk.StringVar()
        )
        self.inputs['Technician'].grid(row=0, column=2)

        # line 2
        self.inputs['Lab'] = LabelInput(
            recordinfo, "Lab",
            input_class=ttk.Combobox,
            input_var=tk.StringVar(),
            input_args={"values": ["A", "B", "C", "D", "E"]}
        )
        self.inputs['Lab'].grid(row=1, column=0)
        self.inputs['Plot'] = LabelInput(
            recordinfo, "Plot",
            input_class=ttk.Combobox,
            input_var=tk.IntVar(),
            input_args={"values": list(range(1, 21))}
        )
        self.inputs['Plot'].grid(row=1, column=1)
        self.inputs['Seed sample'] = LabelInput(
            recordinfo, "Seed sample",
            input_var=tk.StringVar()
        )
        self.inputs['Seed sample'].grid(row=1, column=2)

        recordinfo.grid(row=0, column=0, sticky=(tk.W + tk.E))

        # Environment Data
        environmentinfo = tk.LabelFrame(self, text="Environment Data")
        self.inputs['Humidity'] = LabelInput(
            environmentinfo, "Humidity (g/m³)",
            input_class=tk.Spinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": 0.5, "to": 52.0, "increment": .01}
        )
        self.inputs['Humidity'].grid(row=0, column=0)
        self.inputs['Light'] = LabelInput(
            environmentinfo, "Light (klx)",
            input_class=tk.Spinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": 0, "to": 100, "increment": .01}
        )
        self.inputs['Light'].grid(row=0, column=1)
        self.inputs['Temperature'] = LabelInput(
            environmentinfo, "Tenmperature (°C)",
            input_class=tk.Spinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": 4, "to": 40, "increment": .01}
        )
        self.inputs['Temperature'].grid(row=0, column=2)
        self.inputs['Equipment Fault'] = LabelInput(
            environmentinfo, "Equipment Fault",
            input_class=ttk.Checkbutton,
            input_var=tk.BooleanVar()
        )
        self.inputs['Equipment Fault'].grid(row=1, column=0, columnspan=3)

        environmentinfo.grid(row=1, column=0, sticky=(tk.W + tk.E))

        # Plant Data section
        plantinfo = tk.LabelFrame(self, text="Plant Data")

        self.inputs['Plants'] = LabelInput(
            plantinfo, "Plants",
            input_class=tk.Spinbox,
            input_var=tk.IntVar(),
            input_args={"from_": 0, "to": 20}
        )
        self.inputs['Plants'].grid(row=0, column=0)
        self.inputs['Blossoms'] = LabelInput(
            plantinfo, "Blossoms",
            input_class=tk.Spinbox,
            input_var=tk.IntVar(),
            input_args={"from_": 0, "to": 1000}
        )
        self.inputs['Blossoms'].grid(row=0, column=1)
        self.inputs['Fruit'] = LabelInput(
            plantinfo, "Fruit",
            input_class=tk.Spinbox,
            input_var=tk.IntVar(),
            input_args={"from_": 0, "to": 1000}
        )
        self.inputs['Fruit'].grid(row=0, column=2)

        # Height data
        self.inputs['Min Height'] = LabelInput(
            plantinfo, "Min Height (cm)",
            input_class=tk.Spinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": 0, "to": 1000, "increment": .01}
        )
        self.inputs['Min Height'].grid(row=1, column=0)
        self.inputs['Max Height'] = LabelInput(
            plantinfo, "Max Height (cm)",
            input_class=tk.Spinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": 0, "to": 1000, "increment": .01}
        )
        self.inputs['Max Height'].grid(row=1, column=1)
        self.inputs['Median Height'] = LabelInput(
            plantinfo, "Median Height (cm)",
            input_class=tk.Spinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": 0, "to": 1000, "increment": .01}
        )
        self.inputs['Median Height'].grid(row=1, column=2)

        plantinfo.grid(row=2, column=0, sticky=(tk.W + tk.E))

        # Notes section
        self.inputs['Notes'] = LabelInput(
            self, "Notes",
            input_class=tk.Text,
            input_args={"width": 75, "height": 10}
        )
        self.inputs['Notes'].grid(sticky=tk.W, row=3, column=0)

        # default the form
        self.reset()

    def get(self):
        """Retrieve data from form as a dict"""

        # We need to retrieve the data from Tkinter variables
        # and place it in regular Python objects

        data = {}
        for key, widget in self.inputs.items():
            data[key] = widget.get()
        return data

    def reset(self):
        """Resets the form entries"""

        # clear all values
        for widget in self.inputs.values():
            widget.set('')


class Application(tk.Tk):
    """Application root window"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.title("ABQ Data Entry Application")
        # self.resizable(width=False, height=False)

        ttk.Label(
            self,
            text="ABQ Data Entry Application",
            font=("TkDefaultFont", 16)
        ).grid(row=0)

        self.recordform = DataRecordForm(self)
        self.recordform.grid(row=1, padx=10)

        self.savebutton = ttk.Button(self, text="Save", command=self.on_save)
        self.savebutton.grid(sticky=tk.E, row=2, padx=10)

        # status bar
        self.status = tk.StringVar()
        self.statusbar = ttk.Label(self, textvariable=self.status)
        self.statusbar.grid(sticky=(tk.W + tk.E), row=3, padx=10)

        self.records_saved = 0

    def on_save(self):
        """Handles save button clicks"""

        # For now, we save to a hardcoded filename with a datestring.
        # If it doesnt' exist, create it,
        # otherwise just append to the existing file
        datestring = datetime.today().strftime("%Y-%m-%d")
        filename = "abq_data_record_{}.csv".format(datestring)
        newfile = not os.path.exists(filename)

        data = self.recordform.get()

        with open(filename, 'a') as fh:
            csvwriter = csv.DictWriter(fh, fieldnames=data.keys())
            if newfile:
                csvwriter.writeheader()
            csvwriter.writerow(data)

        self.records_saved += 1
        self.status.set(
            "{} records saved this session".format(self.records_saved))
        self.recordform.reset()


if __name__ == "__main__":

    app = Application()
    app.mainloop()



import tkinter as tk
from tkinter import ttk

"""
这是树形demo
这是树形demo
这是树形demo
这是树形demo
这是树形demo
这是树形demo
这是树形demo
这是树形demo
这是树形demo
这是树形demo
"""


class FileTree:
    def __init__(self, root):
        self.root = root
        self.tree = None

    def create_tree(self):
        # 创建 Treeview
        self.tree = ttk.Treeview(self.root)

        # 添加根节点
        self.tree.insert("", "end", text="根目录", open=True)

        # 添加子节点
        folder1 = self.tree.insert("", "end", text="文件夹1", open=True)
        file11 = self.tree.insert(folder1, "end", text="文件1-1")
        file12 = self.tree.insert(folder1, "end", text="文件1-2")

        folder2 = self.tree.insert("", "end", text="文件夹2", open=True)
        file21 = self.tree.insert(folder2, "end", text="文件2-1")
        file22 = self.tree.insert(folder2, "end", text="文件2-2")

        # 绑定点击事件
        self.tree.bind("<Double-Button-1>", self.toggle_tree)

        # 设置样式
        style = ttk.Style()
        style.configure("Treeview", font=("Arial", 12))

        # 显示 Treeview
        self.tree.pack(expand=True, fill="both")

    def toggle_tree(self, event):
        item_id = self.tree.identify_row(event.y)
        if item_id:
            res = self.tree.item(item_id, open=not self.tree.item(item_id, "open"))
            print(res)


# 创建根窗口和 FileTree 对象
root = tk.Tk()
root.title("可折叠文件树")
file_tree = FileTree(root)

# 创建文件树
file_tree.create_tree()

# root.mainloop()



import requests


class Requester:
    def __init__(self):
        self.base_url = "http://localhost:3000"

    def get(self, url, params=None, headers=None):
        full_url = self.base_url + url
        response = requests.get(full_url, params=params, headers=headers)
        return response.json()

    def post(self, url, data=None, json=None, headers=None):
        full_url = self.base_url + url
        response = requests.post(full_url, data=data, json=json, headers=headers)
        return response.json()



from thonny.sqlite3_sql import ExamService, HomeworkService, QuestionService

#
homework = HomeworkService()
# list1 = homework.list_homework()
homework_score = homework.list_homework_score()
print(homework_score)
# #
# #
# exam = ExamService()
# list2 = exam.list_exam(course_id=2)
# #
# qus = QuestionService()
# # detail = qus.question_detail(1)
# # print(detail)
# exam_answers = {
#     "exam_id": 1,  # 考试ID
#     "stems": [  # 题目集
#         {
#             "stem_id": 1,  # 题目ID
#             "stem_type": "SINGLE_CHOICE",  # 题目类型
#             "stem_score": 10.0,  # 题目分数
#             "answer_ids": "1",  # 答案集
#             "correct_answer_ids": "1"  # 正确答案集
#         }, {
#             "stem_id": 2,  # 题目ID
#             "stem_type": "SINGLE_CHOICE",  # 题目类型
#             "stem_score": 10.0,  # 题目分数
#             "answer_ids": "5",  # 答案集
#             "correct_answer_ids": "6"  # 正确答案集
#         }, {
#             "stem_id": 3,  # 题目ID
#             "stem_type": "SINGLE_CHOICE",  # 题目类型
#             "stem_score": 10.0,  # 题目分数
#             "answer_ids": "10",  # 答案集
#             "correct_answer_ids": "11"  # 正确答案集
#         }
#     ]
# }
# qus.save_exam_answers(exam_answers)
# # homework_answers = {
# #     "homework_id": 1,  # 考试ID
# #     "stems": [  # 题目集
# #         {
# #             "stem_id": 1,  # 题目ID
# #             "stem_type": "SINGLE_CHOICE",  # 题目类型
# #             "stem_score": 10.0,  # 题目分数
# #             "answer_ids": "1",  # 答案集
# #             "correct_answer_ids": "1"  # 正确答案集
# #         }, {
# #             "stem_id": 2,  # 题目ID
# #             "stem_type": "SINGLE_CHOICE",  # 题目类型
# #             "stem_score": 10.0,  # 题目分数
# #             "answer_ids": "5",  # 答案集
# #             "correct_answer_ids": "6"  # 正确答案集
# #         }, {
# #             "stem_id": 3,  # 题目ID
# #             "stem_type": "SINGLE_CHOICE",  # 题目类型
# #             "stem_score": 10.0,  # 题目分数
# #             "answer_ids": "10",  # 答案集
# #             "correct_answer_ids": "11"  # 正确答案集
# #         }
# #     ]
# # }
# # qus.save_homework_answers(homework_answers)
# # submit_count = qus.query_submit_count("EXAM", 1)
# # print(submit_count)
# import tkinter as tk
# from tkinter import messagebox
#
# class CountdownTimer:
#     def __init__(self, total_seconds):
#         self.total_seconds = total_seconds
#         self.hours = 0
#         self.minutes = 0
#         self.seconds = 0
#
#         self.root = tk.Tk()
#         self.root.title("倒计时")
#         self.root.geometry("200x100")
#
#         self.countdown_label = tk.Label(self.root, text="倒计时: ")
#         self.countdown_label.pack()
#
#     def start_countdown(self):
#         if self.total_seconds > 0:
#             self.calculate_time()
#
#             self.countdown_label['text'] = f"倒计时: {self.hours:02d}:{self.minutes:02d}:{self.seconds:02d}"
#
#             self.total_seconds -= 1
#             self.root.after(1000, self.start_countdown)
#         else:
#             messagebox.showinfo("倒计时完成", "时间到了！")
#
#     def calculate_time(self):
#         self.hours = self.total_seconds // 3600
#         self.minutes = (self.total_seconds % 3600) // 60
#         self.seconds = self.total_seconds % 60
#
#     def run(self):
#         self.start_countdown()
#         self.root.mainloop()
#
# # 创建一个倒计时器对象，设定倒计时的总秒数
# timer = CountdownTimer(3666)  # 示例倒计时总秒数
# timer.run()



import logging
import os.path
import sys
import time
from logging import getLogger
from typing import TYPE_CHECKING, List, Optional, cast

from thonny.utils.internet_utils import InternetUtil

from thonny.common import is_private_python, is_virtual_executable
from thonny.login import UserLogin
from thonny.mysql_sql import IUserService
from thonny.sqlite3_sql import UserService

# 定义三个全局变量
_last_module_count = 0
_last_modules = set()

_last_time = time.time()

# _name__ 表示当前模块的名称，通常是运行时的模块名
logger = getLogger(__name__)

"""
新增模块，加载时间报告时间
"""


def report_time(label: str) -> None:
    """
    Method for finding unwarranted imports and delays.
    """
    # 要引用全局变量先用gloabal处理
    global _last_time, _last_module_count, _last_modules

    log_modules = True

    t = time.time()
    # 当前模块导入的数量
    mod_count = len(sys.modules)
    mod_delta = mod_count - _last_module_count
    # 增加的模块数量
    if mod_delta > 0:
        mod_info = f"(+{mod_count - _last_module_count} modules)"
    else:
        mod_info = ""
    logger.info("TIME/MODS %s %s %s", f"{t - _last_time:.3f}", label, mod_info)
    # 记录新增的模块信息到日志中。
    if log_modules and mod_delta > 0:
        current_modules = set(sys.modules.keys())
        # 获取当前所有模块的集合，并打印出新增模块的名称。
        logger.info("NEW MODS %s", list(sorted(current_modules - _last_modules)))
        # 更新全局变量 _last_time、_last_module_count 和 _last_modules 的值。
        _last_modules = current_modules

    _last_time = t
    _last_module_count = mod_count


report_time("After defining report_time")


SINGLE_INSTANCE_DEFAULT = True
BACKEND_LOG_MARKER = "Thonny's backend.log"

"""
获取已知文件夹路径
"""


def _get_known_folder(ID):
    # http://stackoverflow.com/a/3859336/261181
    # http://www.installmate.com/support/im9/using/symbols/functions/csidls.htm
    import ctypes.wintypes

    SHGFP_TYPE_CURRENT = 0
    buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
    # SHGetFolderPathW：这是 Shell32 动态链接库中的一个函数，用于获取指定文件夹的路径。
    """
    0：表示当前窗口句柄（HWND），此处为桌面窗口的句柄。
    ID：表示特定文件夹的标识符（CSIDL），用于指定要获取的文件夹。
    0：表示需要获取文件夹路径的用户的访问权限，默认为当前用户。
    SHGFP_TYPE_CURRENT：表示获取当前文件夹的路径。
    buf：表示用于存储文件夹路径的缓冲区。
    SHGetFolderPathW 函数的作用是根据传入的参数获取特定文件夹的路径，并将路径存储在 buf 缓冲区中。
    该函数在 Windows 系统中非常常用，可以用于获取用户特定文件夹的路径，例如桌面、我的文档、应用程序数据等
    """
    ctypes.windll.shell32.SHGetFolderPathW(0, ID, 0, SHGFP_TYPE_CURRENT, buf)
    assert buf.value
    return buf.value


"""
在 Windows 系统中，ID 为 26 表示 roaming appdata 目录。
"""


def _get_roaming_appdata_dir():
    return _get_known_folder(26)


def _get_local_appdata_dir():
    return _get_known_folder(28)


"""
不同的情况下计算并返回 Thonny 用户目录的路径
"""


def _compute_thonny_user_dir():
    # 获取THONNY_USER_DIR环境变量的值
    if os.environ.get("THONNY_USER_DIR", ""):
        # os.path.expanduser() 函数会将路径中的波浪号 ~ 展开为当前用户的主目录路径。
        return os.path.expanduser(os.environ["THONNY_USER_DIR"])
    elif is_portable():
        if sys.platform == "win32":
            # 前 Python 解释器的可执行文件所在的目录
            root_dir = os.path.dirname(sys.executable)
        elif sys.platform == "darwin":
            root_dir = os.path.join(
                os.path.dirname(sys.executable), "..", "..", "..", "..", "..", ".."
            )
        else:
            root_dir = os.path.join(os.path.dirname(sys.executable), "..")
        #     这是一个用于规范化路径的函数，它将路径中的斜杠、反斜杠等转换成当前操作系统使用的路径分隔符
        return os.path.normpath(os.path.abspath(os.path.join(root_dir, "user_data")))
    elif is_virtual_executable(sys.executable) and not is_private_python(sys.executable):
        return os.path.join(sys.prefix, ".thonny")
    elif sys.platform == "win32":
        return os.path.join(_get_roaming_appdata_dir(), "Thonny")
    elif sys.platform == "darwin":
        return os.path.expanduser("~/Library/Thonny")
    else:
        # https://specifications.freedesktop.org/basedir-spec/latest/ar01s02.html
        data_home = os.environ.get(
            # 这行代码使用波浪号 ~ 展开为当前用户的主目录路径
            # "XDG_CONFIG_HOME"：这是一个环境变量，指定了用户配置文件的默认基础目录路径
            "XDG_CONFIG_HOME",
            os.path.expanduser(os.path.join("~", ".config")),
        )
        return os.path.join(data_home, "Thonny")


def _read_configured_debug_mode():
    if not os.path.exists(CONFIGURATION_FILE):
        return False

    try:
        with open(CONFIGURATION_FILE, encoding="utf-8") as fp:
            for line in fp:
                if "debug_mode" in line and "True" in line:
                    return True
        # 处理文件以后，没有debug_mode的就直接，结束文件处理，返回False
        return False
    except Exception:
        import traceback

        traceback.print_exc()
        return False


"""
可移植的
"""


def is_portable():
    # it can be explicitly declared as portable or shared ...
    # os.path.join(os.path.dirname(sys.executable), "portable_thonny.ini")
    # 将会把 Python 可执行文件所在的目录路径和文件名拼接起来，形成标记文件 "portable_thonny.ini" 的完整路径。
    portable_marker_path = os.path.join(os.path.dirname(sys.executable), "portable_thonny.ini")
    shared_marker_path = os.path.join(os.path.dirname(sys.executable), "shared_thonny.ini")

    if os.path.exists(portable_marker_path) and not os.path.exists(shared_marker_path):
        return True
    elif not os.path.exists(portable_marker_path) and os.path.exists(shared_marker_path):
        return False

    # ... or it becomes implicitly portable if it's on a removable drive
    # __file__：它是一个特殊变量，表示当前脚本文件的路径。
    abs_location = os.path.abspath(__file__)
    if sys.platform == "win32":
        drive = os.path.splitdrive(abs_location)[0]
        if drive.endswith(":"):
            from ctypes import windll

            # 2 表示可移动磁盘类型
            return windll.kernel32.GetDriveTypeW(drive) == 2  # @UndefinedVariable
        else:
            return False
    #     对于 macOS 平台
    elif sys.platform == "darwin":
        # not exact heuristics
        return abs_location.startswith("/Volumes/")
    # 对于其他平台
    else:
        # not exact heuristics
        return abs_location.startswith("/media/") or abs_location.startswith("/mnt/")


_THONNY_VERSION = None


def get_version():
    global _THONNY_VERSION
    if _THONNY_VERSION:
        return _THONNY_VERSION
    try:
        package_dir = os.path.dirname(sys.modules["thonny"].__file__)
        with open(os.path.join(package_dir, "VERSION"), encoding="ASCII") as fp:
            _THONNY_VERSION = fp.read().strip()
            return _THONNY_VERSION
    except Exception:
        return "0.0.0"


THONNY_USER_DIR = _compute_thonny_user_dir()
CONFIGURATION_FILE = os.path.join(THONNY_USER_DIR, "configuration.ini")
_CONFIGURED_DEBUG = _read_configured_debug_mode()


_IPC_FILE = None

"""
获取ipc通信文件的路径
"""


def get_ipc_file_path():
    global _IPC_FILE
    if _IPC_FILE:
        print("1111111" + _IPC_FILE)
        return _IPC_FILE

    if sys.platform == "win32":
        base_dir = _get_local_appdata_dir()
    else:
        base_dir = os.environ.get("XDG_RUNTIME_DIR")
        if not base_dir or not os.path.exists(base_dir):
            base_dir = os.environ.get("TMPDIR")

    if not base_dir or not os.path.exists(base_dir):
        base_dir = THONNY_USER_DIR

    for name in ("LOGNAME", "USER", "LNAME", "USERNAME"):
        if name in os.environ:
            username = os.environ.get(name)
            break
    else:
        username = os.path.basename(os.path.expanduser("~"))

    ipc_dir = os.path.join(base_dir, "thonny-%s" % username)
    os.makedirs(ipc_dir, exist_ok=True)

    if not sys.platform == "win32":
        # 设置 IPC 目录的权限为 0700
        os.chmod(ipc_dir, 0o700)

    _IPC_FILE = os.path.join(ipc_dir, "ipc.sock")
    print("22222" + _IPC_FILE)
    return _IPC_FILE


def _check_welcome():
    from thonny import misc_utils

    if not os.path.exists(CONFIGURATION_FILE) and not misc_utils.running_on_rpi():
        from thonny.config import ConfigurationManager
        from thonny.first_run import FirstRunWindow

        mgr = ConfigurationManager(CONFIGURATION_FILE)

        win = FirstRunWindow(mgr)
        win.mainloop()
        return win.ok
    else:
        return True


def launch():
    # import runpy
    if sys.executable.endswith("thonny.exe"):
        # otherwise some library may try to run its subprocess with thonny.exe
        # NB! Must be pythonw.exe not python.exe, otherwise Runner thinks console
        # is already allocated.
        sys.executable = sys.executable[: -len("thonny.exe")] + "pythonw.exe"
    # Data Processing Installation
    set_dpi_aware()
    """
    @Author ：yb
    @Date   ：2023-08-13 11:28:27
    @Desc   ：It shouldn't work
    """
    # try:
    #     runpy.run_module("thonny.customize", run_name="__main__")
    # except ImportError:
    #     logging.error(ImportError)
    #     pass

    prepare_thonny_user_dir()
    _configure_frontend_logging()

    if not _check_welcome():
        return 0

    # 检查是否进行委托操作
    if _should_delegate():
        try:
            #
            _delegate_to_existing_instance(sys.argv[1:])
            print("666666 Delegated to an existing Thonny instance. Exiting now.")
            return 0
        except Exception:
            import traceback

            traceback.print_exc()

    user_login = UserLogin()
    user_login.create_login_view()
    if not user_login.login_result:
        return 0

    # 检查网络，同步用户数据到远程数据库
    internet_util = InternetUtil()
    connection = internet_util.check_internet_connection()
    if connection:
        user_service = UserService()
        user = user_service.get_one()
        origin_user_id = user[5]
        if origin_user_id is None:
            # 将用户数据同步到远端并更新本地用户远端id
            i_user_service = IUserService()
            origin_user_id = i_user_service.insert_user(user)
            user_id = user[0]
            user_service.update_user(origin_user_id, user_id)
    # Did not or could not delegate
    try:
        from thonny import workbench

        # 建立工作台
        bench = workbench.Workbench()
        bench.mainloop()
        return 0

    except Exception:
        import tkinter as tk
        import traceback
        from logging import exception

        exception("Internal launch or mainloop error")
        from thonny import ui_utils

        dlg = ui_utils.LongTextDialog("Internal error", traceback.format_exc())
        ui_utils.show_dialog(dlg, tk._default_root)
        return -1
    finally:
        runner = get_runner()
        if runner is not None:
            runner.destroy_backend()

    return 0


def prepare_thonny_user_dir():
    # 检查THONNY_USER_DIR是否存在，如果不存在则创建
    if not os.path.exists(THONNY_USER_DIR):
        # 创建目录，并设置其权限为0o700
        """
        0o 表示后面是一个八进制
        所有者（Owner）可以读取（4）、写入（2）和执行（1）该文件或目录；
        所属组（Group）没有任何权限（0）；
        其他用户（Others）也没有任何权限（0）。
        """
        os.makedirs(THONNY_USER_DIR, mode=0o700, exist_ok=True)

        # user_dir_template是根据安装后的需要在多用户环境中提供替代默认用户环境的一种方式
        template_dir = os.path.join(os.path.dirname(__file__), "user_dir_template")

        # 检查template_dir是否是一个目录
        if os.path.isdir(template_dir):
            import shutil

            def copy_contents(src_dir, dest_dir):
                # 复制源目录（src_dir）的内容到目标目录（dest_dir）
                # 我们希望复制的文件和文件夹具有当前用户的权限

                # 遍历源目录中的每个项目（文件/文件夹）
                for name in os.listdir(src_dir):
                    src_item = os.path.join(src_dir, name)  # 源项目的完整路径
                    dest_item = os.path.join(dest_dir, name)  # 目标项目的完整路径
                    # 如果该项目是一个目录，则递归地创建它并复制其内容
                    if os.path.isdir(src_item):
                        os.makedirs(dest_item, mode=0o700)  # 创建目录，并设置其权限为0o700
                        copy_contents(src_item, dest_item)  # 递归复制子目录的内容
                    else:
                        # 如果该项目是一个文件，则将其复制到目标目录，并设置权限为0o600
                        shutil.copyfile(src_item, dest_item)
                        os.chmod(dest_item, 0o600)

            # 开始复制template_dir的内容到THONNY_USER_DIR
            copy_contents(template_dir, THONNY_USER_DIR)


def _should_delegate():
    # 此方法用于判断路径不是存在的
    if not os.path.exists(get_ipc_file_path()):
        # no previous instance
        print("88888" + "no previous instance")
        return False

    from thonny.config import try_load_configuration

    configuration_manager = try_load_configuration(CONFIGURATION_FILE)
    configuration_manager.set_default("general.single_instance", SINGLE_INSTANCE_DEFAULT)
    print("7777777 " + str(configuration_manager.get_option("general.single_instance")))

    return configuration_manager.get_option("general.single_instance")


def _delegate_to_existing_instance(args):
    import socket

    from thonny import workbench

    transformed_args = []
    for arg in args:
        if not arg.startswith("-"):
            arg = os.path.abspath(arg)

        transformed_args.append(arg)

    try:
        sock, secret = _create_client_socket()
    except Exception:
        # Maybe the lock is abandoned or the content is corrupted
        try:
            os.remove(get_ipc_file_path())
        except Exception:
            import traceback

            traceback.print_exc()
        raise

    data = repr((secret, transformed_args)).encode(encoding="utf_8")

    sock.settimeout(2.0)
    sock.sendall(data)
    sock.shutdown(socket.SHUT_WR)
    response = bytes([])
    while len(response) < len(workbench.SERVER_SUCCESS):
        new_data = sock.recv(2)
        if len(new_data) == 0:
            break
        else:
            response += new_data

    if response.decode("UTF-8") != workbench.SERVER_SUCCESS:
        raise RuntimeError("Unsuccessful delegation")


def _create_client_socket():
    import socket

    timeout = 2.0

    if sys.platform == "win32":
        with open(get_ipc_file_path(), "r") as fp:
            port = int(fp.readline().strip())
            secret = fp.readline().strip()

        # "localhost" can be much slower than "127.0.0.1"
        client_socket = socket.create_connection(("127.0.0.1", port), timeout=timeout)
    else:
        client_socket = socket.socket(socket.AF_UNIX)  # @UndefinedVariable
        client_socket.settimeout(timeout)
        client_socket.connect(get_ipc_file_path())
        secret = ""

    return client_socket, secret


def _configure_frontend_logging() -> None:
    _configure_logging(get_frontend_log_file(), _choose_logging_level())


def configure_backend_logging() -> None:
    _configure_logging(get_backend_log_file(), None)


def get_backend_log_file():
    return os.path.join(THONNY_USER_DIR, "backend.log")


def get_frontend_log_file():
    return os.path.join(THONNY_USER_DIR, "frontend.log")


def _get_orig_argv() -> Optional[List[str]]:
    try:
        from sys import orig_argv  # since 3.10

        return sys.orig_argv
    except ImportError:
        # https://stackoverflow.com/a/57914236/261181
        import ctypes

        argc = ctypes.c_int()
        argv = ctypes.POINTER(ctypes.c_wchar_p if sys.version_info >= (3,) else ctypes.c_char_p)()
        try:
            ctypes.pythonapi.Py_GetArgcArgv(ctypes.byref(argc), ctypes.byref(argv))
        except AttributeError:
            # See https://github.com/thonny/thonny/issues/2206
            # and https://bugs.python.org/issue40910
            # This symbol is not available in thonny.exe built agains Python 3.8
            return None

        # Ctypes are weird. They can't be used in list comprehensions, you can't use `in` with them, and you can't
        # use a for-each loop on them. We have to do an old-school for-i loop.
        arguments = list()
        for i in range(argc.value):
            arguments.append(argv[i])

        return arguments


def _configure_logging(log_file, console_level=None):
    logFormatter = logging.Formatter(
        "%(asctime)s.%(msecs)03d [%(threadName)s] %(levelname)-7s %(name)s: %(message)s", "%H:%M:%S"
    )

    file_handler = logging.FileHandler(log_file, encoding="UTF-8", mode="w")
    file_handler.setFormatter(logFormatter)

    main_logger = logging.getLogger("thonny")
    contrib_logger = logging.getLogger("thonnycontrib")
    pipkin_logger = logging.getLogger("pipkin")

    # NB! Don't mess with the main root logger, because (CPython) backend runs user code
    for logger in [main_logger, contrib_logger, pipkin_logger]:
        logger.setLevel(_choose_logging_level())
        logger.propagate = False  # otherwise it will be also reported by IDE-s root logger
        logger.addHandler(file_handler)

    if console_level is not None:
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(logFormatter)
        console_handler.setLevel(console_level)
        for logger in [main_logger, contrib_logger]:
            logger.addHandler(console_handler)

    # Log most important info as soon as possible
    main_logger.info("Thonny version: %s", get_version())
    main_logger.info("cwd: %s", os.getcwd())
    main_logger.info("original argv: %s", _get_orig_argv())
    main_logger.info("sys.executable: %s", sys.executable)
    main_logger.info("sys.argv: %s", sys.argv)
    main_logger.info("sys.path: %s", sys.path)
    main_logger.info("sys.flags: %s", sys.flags)

    import faulthandler

    fault_out = open(os.path.join(THONNY_USER_DIR, "frontend_faults.log"), mode="w")
    faulthandler.enable(fault_out)


def get_user_base_directory_for_plugins() -> str:
    return os.path.join(THONNY_USER_DIR, "plugins")


def get_sys_path_directory_containg_plugins() -> str:
    from thonny.misc_utils import get_user_site_packages_dir_for_base

    return get_user_site_packages_dir_for_base(get_user_base_directory_for_plugins())


def set_dpi_aware():
    # https://stackoverflow.com/questions/36134072/setprocessdpiaware-seems-not-to-work-under-windows-10
    # https://bugs.python.org/issue33656
    # https://msdn.microsoft.com/en-us/library/windows/desktop/dn280512(v=vs.85).aspx
    # https://github.com/python/cpython/blob/master/Lib/idlelib/pyshell.py
    if sys.platform == "win32":
        try:
            import ctypes

            PROCESS_SYSTEM_DPI_AWARE = 1
            ctypes.OleDLL("shcore").SetProcessDpiAwareness(PROCESS_SYSTEM_DPI_AWARE)
        except (ImportError, AttributeError, OSError):
            pass


if TYPE_CHECKING:
    # Following imports are required for MyPy
    # http://mypy.readthedocs.io/en/stable/common_issues.html#import-cycles
    import thonny.workbench
    from thonny.running import Runner
    from thonny.shell import ShellView
    from thonny.workbench import Workbench

_workbench = None  # type: Optional[Workbench]


def get_workbench() -> "Workbench":
    return cast("Workbench", _workbench)


_runner = None  # type: Optional[Runner]


def set_logging_level(level=None):
    if level is None:
        level = _choose_logging_level()

    logging.getLogger("thonny").setLevel(level)


def _choose_logging_level():
    if in_debug_mode():
        return logging.DEBUG
    else:
        return logging.INFO


def in_debug_mode() -> bool:
    # Value may be something other than string when it is set in Python code
    return (
        os.environ.get("THONNY_DEBUG", False)
        in [
            "1",
            1,
            "True",
            True,
            "true",
        ]
        or _CONFIGURED_DEBUG
    )


def get_runner() -> "Runner":
    return cast("Runner", _runner)


def get_shell() -> "ShellView":
    return cast("ShellView", get_workbench().get_view("ShellView"))


report_time("After loading thonny module")



import json
import requests

from thonny.utils.HttpUtils import HttpUtil

"""
曙光接口工具类
"""


# 提交作业参数封装类
class SubmitJobForm:
    def __init__(self):
        self.strJobManagerID = None
        self.mapAppJobInfo = MapAppJobInfo()


# 提交作业详细参数封装类
class MapAppJobInfo:
    def __init__(self):
        self.GAP_WORK_DIR = None
        self.GAP_STD_ERR_FILE = None
        self.GAP_STD_OUT_FILE = None
        self.GAP_APPNAME = None
        self.GAP_WALL_TIME = None
        self.GAP_NPROC = None
        self.GAP_QUEUE = None
        self.GAP_NNODE = None
        self.GAP_SUBMIT_TYPE = None
        self.GAP_NDCU = None
        self.GAP_NGPU = None
        self.GAP_CMD_FILE = None
        self.GAP_JOB_NAME = None


# 提交作业参数封装json
class SubmitJobFormEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, SubmitJobForm):
            return {
                'strJobManagerID': obj.strJobManagerID,
                'mapAppJobInfo': obj.mapAppJobInfo.__dict__
            }
        return super().default(obj)


# =====================以下为曙光接口======================================
# 获取曙光令牌方法
def get_tokens(user, password):
    # 参数校验
    if not user:
        raise ValueError("Invalid user")
    if not password:
        raise ValueError("Invalid password")
    url = "https://ac.sugon.com/ac/openapi/v2/tokens"
    headers = {
        'user': user,
        'password': password,
        'orgId': "4af12268f09383b061bb3c9f0c2d75e9"
    }
    try:
        response = HttpUtil.post(url, headers=headers)
        response_data = json.loads(response.text)
        if response_data['code'] == "0":
            data_list = response_data['data']
            for data in data_list:
                if data['clusterId'] == "11250":
                    token = data['token']
                    return token
        else:
            raise ValueError("Failed to fetch tokens")
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 获取授权区域地址
def get_enable_url(token, url_type):
    """
      url_type参数
          efileUrls   获取 文件传输 相关授权区域地址
          hpcUrls     获取 作业管理相 关授权区域地址
          aiUrls      获取 容器服务 相关授权区域地址
          eshellUrls  获取 ESHELL服务 相关授权区域地址
    """
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not url_type:
        raise ValueError("Invalid url_type")
    if url_type not in ["efileUrls", "hpcUrls", "aiUrls", "eshellUrls"]:
        raise ValueError("Invalid url_type")

    url = "https://ac.sugon.com/ac/openapi/v2/center"
    headers = {'token': token}

    try:
        response = HttpUtil.get(url, headers=headers)
        response_data = json.loads(response.text)
        if response_data['code'] == "0":
            data_urls = response_data['data']
            enable_urls = data_urls.get(url_type, [])
            for enable_url in enable_urls:
                if enable_url['enable'] == "true":
                    return enable_url['url']
            print("No available authorization region")
        else:
            raise ValueError("Failed to fetch enable urls")
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 查询提交作业区域信息 调度器ID
def get_job_cluster(token, hpc_prefix):
    # 参数校验
    if not hpc_prefix:
        raise ValueError("Invalid hpc_prefix")
    if not token:
        raise ValueError("Invalid token")
    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/cluster"
    headers = {
        "token": token
    }

    try:
        response = HttpUtil.get(url, headers=headers)
        print(response.text)
        response.raise_for_status()
        response_data = json.loads(response.text)
        if response_data['code'] == "0":
            data_list = response_data['data']
            for data in data_list:
                job_cluster = data['id']
                return job_cluster
        else:
            raise ValueError("Failed to fetch job_cluster")
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 查询提交作业队列资源信息 队列ID
def get_job_queues(token, hpc_prefix, user, job_cluster):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not hpc_prefix:
        raise ValueError("Invalid hpc_prefix")
    if not user:
        raise ValueError("Invalid user")
    if not job_cluster:
        raise ValueError("Invalid job_cluster")
    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/queuenames/users/" + user
    headers = {
        "token": token
    }
    params = {
        "strJobManagerID": job_cluster
    }
    try:
        response = HttpUtil.get(url, headers=headers, params=params)
        response.raise_for_status()
        print(response.text)
        response_data = json.loads(response.text)
        if response_data['code'] == "0":
            data_list = response_data['data']
            for data in data_list:
                #  kshcnormal cpu共享型队列
                #  kshdexclu17 dpu独占型队列
                #  kshdnormal dpu共享型队列
                if data['queueName'] == "kshcnormal":
                    job_queues = data['id']
                    return job_queues
        else:
            raise ValueError("Failed to fetch job_cluster")
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 提交作业
def submit_job(token, hpc_prefix, cluster, queue, job_cmd, job_name, job_work_dir):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")

    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/apptemplates/BASIC/BASE/job"
    submitJobForm = SubmitJobForm()
    submitJobForm.strJobManagerID = int(cluster)  # 通过getCluster接口去获取调度器ID
    jobInfo = MapAppJobInfo()
    jobInfo.GAP_WORK_DIR = "/work/home/acsx6nwsm6" if job_work_dir == '' else job_work_dir
    jobInfo.GAP_STD_ERR_FILE = "/work/home/acsx6nwsm6" + "/std.err.%j" if job_work_dir == '' else job_work_dir + "/std.err.%j"
    jobInfo.GAP_STD_OUT_FILE = "/work/home/acsx6nwsm6" + "/std.out.%j" if job_work_dir == '' else job_work_dir + "/std.out.%j"
    jobInfo.GAP_APPNAME = "BASE"
    jobInfo.GAP_WALL_TIME = "24:00:00"
    jobInfo.GAP_NPROC = "1"
    jobInfo.GAP_QUEUE = queue
    jobInfo.GAP_NNODE = "1"
    jobInfo.GAP_SUBMIT_TYPE = "cmd"
    jobInfo.GAP_NDCU = ""
    jobInfo.GAP_NGPU = ""
    jobInfo.GAP_CMD_FILE = job_cmd
    jobInfo.GAP_JOB_NAME = job_name

    submitJobForm.mapAppJobInfo = jobInfo
    headers = {
        'token': token,
        'Content-Type': "application/json",
        'User-Agent': "PostmanRuntime/7.15.2",
        'Accept': "*/*",
        'Host': "ai111.hpccube.com:65061",
        'Accept-Encoding': "gzip, deflate",
        'Content-Length': "543",
        'Connection': "keep-alive"
    }
    json_str = json.dumps(submitJobForm, cls=SubmitJobFormEncoder)
    print(json_str)
    try:
        response = HttpUtil.post(url, headers=headers, data=json_str)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 查看文件
def view_files(token, hpc_prefix, dir_path):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not hpc_prefix:
        raise ValueError("Invalid hpc_prefix")
    if not dir_path:
        raise ValueError("Invalid dir_path")
    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/file/content"
    getContent_map = {
        "hostName": "",
        "dirPath": dir_path,
        "triggerNum": 1,
        "rollDirection": "UP"
    }

    headers = {
        "token": token
    }

    try:
        response = HttpUtil.post(url, headers=headers, data=getContent_map)
        response.raise_for_status()
        print("___文件查看结果输出: " + response.text)
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 查询当前作业列表
def view_current_job_list(token, hpc_prefix, job_cluster, job_queues=None, user_name=None, job_id=None,
                          job_name=None, start=None, limit=None, job_stat=None):
    """
    token : token 必填
    hpc_prefix : 作业授权地址 必填
    job_cluster : 作业调度器ID 必填
    job_queues : 作业队列名称
    user_name : 用户名
    job_id : 作业ID
    job_name : 作业名称
    start : 起始坐标  0
    limit : 请求一次获取数据的数目  25
    job_stat : 作业运行状态  # 'statR(运行)','statQ(排队)','statH(保留)','statS(挂起)','statE(退出)','statC(完成)','statW(等待)','statX(其他)'
    """
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not hpc_prefix:
        raise ValueError("Invalid hpc_prefix")
    if not job_cluster:
        raise ValueError("Invalid job_cluster")
    if not start:
        start = 0
    if not limit:
        limit = 25
    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/jobs"
    params = {
        "strClusterIDList": job_cluster,  # 调度器ID  必填
        "strJobOwner": user_name,  # 用户名
        "strJobName": job_name,  # 作业名称
        "strJobId": job_id,  # 作业ID
        "start": start,  # 起始坐标
        "limit": limit,  # 请求一次获取数据的数目
        "strQueueName": job_queues,  # 队列名称
        "strJobStat": job_stat,  # 作业运行状态
    }
    headers = {
        "token": token
    }

    try:
        response = HttpUtil.get(url, headers=headers, params=params)
        print(response)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 查询历史作业列表
def view_history_job_list(token, hpc_prefix, job_cluster, start_time, end_time, job_queues=None, user_name=None,
                          job_id=None, job_name=None, start=None, limit=None, job_stat=None, time_type=None,
                          app_type=None, sort=None, order_by=None, host_name=None, is_query_by_queue_time=None):
    """
     token : token 必填
     hpc_prefix : 作业授权地址 必填
     job_cluster : 作业调度器ID 必填
     start_time : 开始时间 必填  2023-08-15 00:00:00
     end_time : 结束时间 必填  2023-08-21 23:59:59
     job_queues : 作业队列名称
     user_name : 用户名
     job_id : 作业ID
     job_name : 作业名称
     start : 起始坐标  0
     limit : 请求一次获取数据的数目  25
     job_stat : 作业运行状态  # 'statR(运行)','statQ(排队)','statH(保留)','statS(挂起)','statE(退出)','statC(完成)','statW(等待)','statX(其他)'
     time_type : CUSTOM
     app_type : 应用名称
     sort : 排序规则   DESC/ASC
     order_by : 排序字段 	如：jobId
     host_name : 节点名称
     is_query_by_queue_time : 按照结束时间查询false/按照入队时间查询true（推荐false）
     """
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not hpc_prefix:
        raise ValueError("Invalid hpc_prefix")
    if not job_cluster:
        raise ValueError("Invalid job_cluster")
    if not start_time:
        raise ValueError("Invalid start_time")
    if not end_time:
        raise ValueError("Invalid end_time")
    if not start:
        start = 0
    if not limit:
        limit = 25
    if not time_type:
        time_type = "CUSTOM"
    if not is_query_by_queue_time:
        is_query_by_queue_time = "false"
    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/historyjobs"
    params = {
        "strClusterNameList": job_cluster,  # 调度器ID
        "startTime": start_time,  # 开始时间 2021-11-23 01:01:01
        "endTime": end_time,  # 结束时间 2021-11-23 01:01:01
        "timeType": time_type,  # CUSTOM
        "queue": job_queues,  # 队列名称  非必填
        "appType": app_type,  # 应用名称  非必填
        "sort": sort,  # 排序规则  非必填
        "orderBy": order_by,  # 排序字段  非必填
        "jobId": job_id,  # 作业ID  非必填
        "jobState": job_stat,  # 运行状态  非必填
        # 'statR(运行)','statQ(排队)','statH(保留)','statS(挂起)','statE(退出)','statC(完成)','statW(等待)','statX(其他)'
        "hostName": host_name,  # 节点名称  非必填
        "strUser": user_name,  # 用户名称  非必填
        "jobName": job_name,  # 作业名称  非必填
        "start": start,  # 起始坐标
        "limit": limit,  # 请求一次获取数据的数目
        "isQueryByQueueTime": is_query_by_queue_time,  # 按照结束时间查询false/按照入队时间查询true（推荐false）
    }
    headers = {
        "token": token
    }

    try:
        response = HttpUtil.get(url, headers=headers, params=params)
        print(response)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 查询作业详细信息(只能查当前作业列表)
def search_current_job_detail(token, hpc_prefix, job_id):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not hpc_prefix:
        raise ValueError("Invalid efile_prefix")
    if not job_id:
        raise ValueError("Invalid job_id")

    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/jobs/" + job_id
    headers = {
        "token": token
    }

    try:
        response = HttpUtil.get(url, headers=headers)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 查询历史作业详细信息(只能查历史作业列表)
def search_history_job_detail(token, hpc_prefix, job_id, jobmanager_id, acct_time=None):
    """
    job_id : 作业ID
    jobmanager_id : 调度器ID
    acct_time : 入账时间（结束时间），建议传入，能够提升查询性能
    """
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not hpc_prefix:
        raise ValueError("Invalid efile_prefix")
    if not job_id:
        raise ValueError("Invalid job_id")
    if not jobmanager_id:
        raise ValueError("Invalid jobmanager_id")
    if not acct_time:
        acct_time=""

    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/historyjobs/" + jobmanager_id + "/" + job_id + "?acctTime=" + acct_time
    headers = {
        "token": token
    }

    try:
        response = HttpUtil.get(url, headers=headers)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 删除作业(可批量)
def delete_job(token, hpc_prefix, job_info_map):
    """
    job_info_map : 待删除的作业信息，格式为调度器ID,用户名:作业号:; 如：1638523853,test:197:;1638523853,test:196:
    """
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not hpc_prefix:
        raise ValueError("Invalid hpc_prefix")
    if not job_info_map:
        raise ValueError("Invalid job_info_map")
    # 参数封装
    url = hpc_prefix + "/hpc/openapi/v2/jobs"
    params = {
        "jobMethod": "5",  # 作业操作类型，删除类型为5
        "strJobInfoMap": job_info_map
    }
    headers = {
        "token": token
    }

    try:
        response = HttpUtil.delete(url, headers=headers, params=params)
        print(response)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 获取文件
def search_files(token, efile_prefix, limit, start, key_word, path):
    # 参数校验
    if not efile_prefix:
        raise ValueError("Invalid efile_prefix")
    if not token:
        raise ValueError("Invalid token")
    # 参数封装
    url = efile_prefix + "/openapi/v2/file/list"
    params = {
        "limit": limit,
        "order": "desc",
        "orderBy": "lastModifiedTime",
        "path": path,
        "start": start,
        "keyWord": key_word
    }

    headers = {
        "token": token
    }

    try:
        response = HttpUtil.get(url, headers=headers, params=params)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 创建文件夹/文件
def mkdir(token, efile_prefix, path, create_parents):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not efile_prefix:
        raise ValueError("Invalid efile_prefix")
    if not path:
        raise ValueError("Invalid path")
    if not create_parents:
        create_parents = "false"

    # 参数封装
    # path, # 文件绝对路径
    # createParents  # 父目录不存在时是否创建，非必选，可选值：true:是；false:否,默认false
    url = efile_prefix + "/openapi/v2/file/mkdir?" + "path=" + path + "&createParents=" + create_parents
    headers = {
        "token": token
    }
    try:
        response = HttpUtil.post(url, headers=headers)
        response.raise_for_status()
        print("___创建目录结果: " + response.text)
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 文件上传
def upload_file(token, efile_prefix, path, cover, file):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not efile_prefix:
        raise ValueError("Invalid efile_prefix")
    if not path:
        raise ValueError("Invalid path")
    if not cover:
        cover = "uncover"
    if not file:
        raise ValueError("Invalid file")
    # 参数封装
    url = efile_prefix + "/openapi/v2/file/upload"
    files = {'file': open(file, 'rb')}
    print(files)
    upload_map = {
        # 文件路径
        "path": path,
        # cover:强制覆盖，uncover:不覆盖，默认：uncover
        "cover": cover,
    }
    headers = {
        "token": token
    }
    try:
        response = HttpUtil.post(url, headers=headers, data=upload_map, file=files)
        print(response)
        response.raise_for_status()
        print("upload_file: " + response.text)
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 判断文件/文件夹是否存在
def exist_file(token, efile_prefix, path):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not efile_prefix:
        raise ValueError("Invalid efile_prefix")
    if not path:
        raise ValueError("Invalid path")
    # 参数封装
    url = efile_prefix + "/openapi/v2/file/exist"
    exist_map = {
        # 文件/文件夹路径
        "path": path
    }
    headers = {
        "token": token
    }
    try:
        response = HttpUtil.post(url, headers=headers, data=exist_map)
        print(response)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e


# 文件/文件夹删除
def remove_file(token, efile_prefix, path, recursive):
    # 参数校验
    if not token:
        raise ValueError("Invalid token")
    if not efile_prefix:
        raise ValueError("Invalid efile_prefix")
    if not path:
        raise ValueError("Invalid path")
    if not recursive:
        recursive = "false"
    # 参数封装
    # 是否递归删除，非必选，可选值：true:是；false:否，默认false
    url = efile_prefix + "/openapi/v2/file/remove?" + "paths=" + path + "&recursive=" + recursive
    headers = {
        "token": token
    }
    try:
        response = HttpUtil.post(url, headers=headers)
        print(response)
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as e:
        raise Exception("Failed to make a request to the server") from e



import os
import sqlite3
from tkinter import messagebox

import requests

"""
sqlite单机数据库sql语句
NULL: 用于存储空值。

INTEGER: 用于存储整数，根据大小可分为：

TINYINT: 8 位整数
SMALLINT: 16 位整数
MEDIUMINT: 24 位整数
INT 或 INTEGER: 32 位整数
BIGINT: 64 位整数
REAL: 用于存储浮点数（双精度浮点数）。

VARCHAR(255): 用于存储文本字符串。

BLOB: 用于存储二进制数据，如图像、音频、视频等。

NUMERIC: 用于存储数字，可以包括整数和浮点数。
"""

conn = sqlite3.connect("../database.db")
cursor = conn.cursor()
# 定义表名
# 用户
user_table = "t_user"
# 视频
video_table = "t_video"
# 课程
course_table = "t_course"
# 课节
chapter_table = "t_chapter"
# 课时
section_table = "t_section"
# 作业
homework_table = "t_homework"
# 学员作业
homework_answer_log_table = "t_homework_answer_log"
# 考试
exam_table = "t_exam"
# 学员考试
exam_answer_log_table = "t_exam_answer_log"
# 题组
question_table = "t_question"
# 题目
stem_table = "t_stem"
# 题组题目
question_stem_table = "t_question_stem"
# 题目答案
answer_table = "t_answer"
# 用户成绩
user_score_table = "t_user_score"
# 用户课程
user_course_table = "t_user_course"


# 检查是否需要表初始化
def check_need_init():
    query = f"SELECT name FROM sqlite_master WHERE type='table' AND name='{user_table}'"
    cursor.execute(query)
    result = cursor.fetchone()
    if result is None and not result:
        return True
    else:
        return False


def data_init(account, password, user_type, course_type):
    if not check_need_init():
        return
    # 初始化数据库表
    # 用户
    create_user_table_query = f"CREATE TABLE '{user_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, account VARCHAR(255), password VARCHAR(255), user_type VARCHAR(255), login_status INTEGER, origin_user_id INTEGER, version INTEGER)"
    cursor.execute(create_user_table_query)

    # 用户课程
    create_user_table_query = f"CREATE TABLE '{user_course_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, user_id INTEGER, course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_user_table_query)

    # 课程
    create_course_table_query = (
        f"CREATE TABLE '{course_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, title VARCHAR(255), course_describe VARCHAR(255), teacher_describe VARCHAR(255), price REAL, "
        f"section_num INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    )
    cursor.execute(create_course_table_query)
    # 课节
    create_chapter_table_query = f"CREATE TABLE '{chapter_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, title VARCHAR(255), course_id INTEGER, sort INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_chapter_table_query)
    # 课时
    create_section_table_query = f"CREATE TABLE '{section_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, title VARCHAR(255), chapter_id INTEGER, sort INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_section_table_query)
    # 视频
    create_video_table_query = f"CREATE TABLE '{video_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, name VARCHAR(255), path VARCHAR(255), section_id INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_video_table_query)
    # 视频路径检查
    # 题目
    create_stem_table_query = f"CREATE TABLE '{stem_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, content VARCHAR(255), `type` VARCHAR(255), score REAL, analysis VARCHAR(255), `level` TINYINT, enable INTEGER, is_delete INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_stem_table_query)
    # 题组
    create_question_table_query = f"CREATE TABLE '{question_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, title VARCHAR(255), type VARCHAR(255),enable INTEGER,is_delete INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_question_table_query)
    # 题目题组
    create_question_stem_table_query = f"CREATE TABLE '{question_stem_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, question_id INTEGER, stem_id INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_question_stem_table_query)
    # 题目答案
    create_answer_query = f"CREATE TABLE '{answer_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, content VARCHAR(255), stem_id INTEGER, is_correct INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    cursor.execute(create_answer_query)
    # 考试信息
    create_exam_table_query = (
        f"CREATE TABLE '{exam_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, name VARCHAR(255), submit_count INTEGER, display_answer_rule VARCHAR(255), answer_time INTEGER,"
        f" course_id INTEGER,description VARCHAR(255), question_id INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    )
    cursor.execute(create_exam_table_query)
    # 作业信息
    create_homework_table_query = (
        f"CREATE TABLE '{homework_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, name VARCHAR(255), submit_count INTEGER, display_answer_rule VARCHAR(255), answer_time INTEGER,"
        f" section_id INTEGER,description VARCHAR(255), question_id INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    )
    cursor.execute(create_homework_table_query)
    # 学员作业
    create_homework_answer_log_table_query = (
        f"CREATE TABLE '{homework_answer_log_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, user_id INTEGER, homework_id INTEGER, stem_id INTEGER,"
        f" answer_ids VARCHAR(255), correct_answer_ids VARCHAR(255), user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    )
    cursor.execute(create_homework_answer_log_table_query)
    # 学员考试
    create_exam_answer_log_table_query = (
        f"CREATE TABLE '{exam_answer_log_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, user_id INTEGER, exam_id INTEGER, stem_id INTEGER, answer_ids VARCHAR(255),"
        f" correct_answer_ids VARCHAR(255), user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    )
    cursor.execute(create_exam_answer_log_table_query)
    # 学员考试
    create_user_score_table_query = (
        f"CREATE TABLE '{user_score_table}' (id INTEGER PRIMARY KEY AUTOINCREMENT, user_id INTEGER, type VARCHAR(255),"
        f" source_id INTEGER, score REAL, submit_count INTEGER, user_type VARCHAR(255), course_type VARCHAR(255), version INTEGER)"
    )
    cursor.execute(create_user_score_table_query)
    conn.commit()
    # 数据插入
    # 用户数据
    insert_user_sql = f"INSERT INTO '{user_table}' (id, account, password, user_type, login_status, version) VALUES (?, ?, ?, ?, ?, ?)"
    user_values = (1, account, password, user_type, 1, 0)
    cursor.execute(insert_user_sql, user_values)
    conn.commit()
    # 用户课程
    insert_user_course_sql = (
        f"INSERT INTO '{user_course_table}' (id, user_id, course_type, version) VALUES (?, ?, ?, ?)"
    )
    user_course_values = (1, 1, course_type, 0)
    cursor.execute(insert_user_course_sql, user_course_values)
    conn.commit()
    # 课程初始化
    insert_course_sql = f"INSERT INTO '{course_table}' (id, title, course_describe, teacher_describe, price, section_num, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)"
    course_values = (1, course_type, "课程介绍", "主讲教师介绍", 300.00, 10, user_type, course_type, 0)
    cursor.execute(insert_course_sql, course_values)
    conn.commit()
    if course_type == "PYTHON" and user_type == "PRIMARY_AND_SECONDARY":
        # 课节初始化
        insert_chapter_sql = f"INSERT INTO '{chapter_table}' (id ,title, course_id, sort, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?)"
        chapter_values = [
            (1, "第一课节", 1, 0, user_type, course_type, 0),
            (2, "第二课节", 1, 0, user_type, course_type, 0),
            (3, "第三课节", 1, 0, user_type, course_type, 0),
        ]
        cursor.executemany(insert_chapter_sql, chapter_values)
        conn.commit()

        # 课时初始化
        insert_chapter_sql = f"INSERT INTO '{section_table}' (id, title, chapter_id, sort, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?)"
        chapter_values = [
            (1, "第一课节-第一课时", 1, 0, user_type, course_type, 0),
            (2, "第一课节-第二课时", 1, 0, user_type, course_type, 0),
            (3, "第一课节-第三课时", 1, 0, user_type, course_type, 0),
            (4, "第二课节-第一课时", 2, 0, user_type, course_type, 0),
            (5, "第二课节-第二课时", 2, 0, user_type, course_type, 0),
            (6, "第二课节-第三课时", 2, 0, user_type, course_type, 0),
        ]
        cursor.executemany(insert_chapter_sql, chapter_values)
        conn.commit()

        # 题目初始化
        insert_stem_sql = f"INSERT INTO '{stem_table}' (id, content, `type`, score,analysis, `level`, enable, is_delete, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
        stem_values = [
            (1, "第1个题目", "SINGLE_CHOICE", 10.00, "题目答案解析", 2, 1, 0, user_type, course_type, 0),
            (2, "第2个题目", "MULTIPLE_CHOICE", 10.00, "题目答案解析", 2, 1, 0, user_type, course_type, 0),
            (3, "第3个题目", "JUDGE", 10.00, "题目答案解析", 2, 1, 0, user_type, course_type, 0),
            (4, "第4个题目", "OTHER", 10.00, "题目答案解析", 2, 1, 0, user_type, course_type, 0),
        ]
        cursor.executemany(insert_stem_sql, stem_values)
        conn.commit()
        # 答案初始化
        insert_answer_sql = f"INSERT INTO '{answer_table}' (id, content, stem_id, is_correct, user_type, course_type,version) VALUES (?, ?, ?, ?, ?, ?, ?)"
        answer_values = [
            (1, "单选答案1", 1, 1, user_type, course_type, 0),
            (2, "单选答案2", 1, 0, user_type, course_type, 0),
            (3, "单选答案3", 1, 0, user_type, course_type, 0),
            (4, "单选答案4", 1, 0, user_type, course_type, 0),
            (5, "多选答案1", 2, 1, user_type, course_type, 0),
            (6, "多选答案2", 2, 1, user_type, course_type, 0),
            (7, "多选答案3", 2, 0, user_type, course_type, 0),
            (8, "多选答案4", 2, 0, user_type, course_type, 0),
            (9, "正确", 3, 1, user_type, course_type, 0),
            (10, "错误", 3, 0, user_type, course_type, 0),
            (11, "正确", 0, 0, user_type, course_type, 0),
            (12, "错误", 0, 1, user_type, course_type, 0),
            (13, "其他答案4", 4, 1, user_type, course_type, 0),
            (14, "其他答案4", 4, 0, user_type, course_type, 0),
        ]
        cursor.executemany(insert_answer_sql, answer_values)
        conn.commit()

        # 题组初始化
        insert_question_sql = f"INSERT INTO '{question_table}' (id, title, type, enable,is_delete, user_type, course_type ,version) VALUES (?, ?, ?, ?, ?, ?, ?, ?)"
        question_values = [
            (1, "作业题组", "HOMEWORK", 1, 0, user_type, course_type, 0),
            (2, "考试题组", "EXAM", 1, 0, user_type, course_type, 0),
        ]
        cursor.executemany(insert_question_sql, question_values)
        conn.commit()

        # 题组题目绑定
        insert_question_stem_sql = f"INSERT INTO '{question_stem_table}' (id, question_id, stem_id, user_type, course_type,version) VALUES (?, ?, ?, ?, ?, ?)"
        question_stem_values = [
            (1, 1, 1, user_type, course_type, 0),
            (2, 1, 2, user_type, course_type, 0),
            (3, 1, 3, user_type, course_type, 0),
            (4, 1, 4, user_type, course_type, 0),
            (5, 2, 1, user_type, course_type, 0),
            (6, 2, 2, user_type, course_type, 0),
            (7, 2, 3, user_type, course_type, 0),
            (8, 2, 4, user_type, course_type, 0),
        ]
        cursor.executemany(insert_question_stem_sql, question_stem_values)
        conn.commit()

        # 初始化作业
        insert_homework_sql = f"INSERT INTO '{homework_table}' (id, name, submit_count, display_answer_rule, answer_time, section_id, description, question_id, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
        homework_values = (1, "课时作业", 3, "DISPLAY", 3600, 1, "作业介绍", 1, user_type, course_type, 0)
        cursor.execute(insert_homework_sql, homework_values)
        conn.commit()

        # 初始化考试
        insert_exam_sql = f"INSERT INTO '{exam_table}' (id, name, submit_count, display_answer_rule, answer_time, course_id, description, question_id, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
        exam_values = (1, "课程考试", 3, "DISPLAY", 3600, 1, "考试介绍", 1, user_type, course_type, 0)
        cursor.execute(insert_exam_sql, exam_values)
        conn.commit()

        # 初始化视频
        insert_video_sql = f"INSERT INTO '{video_table}' (id, name, path, section_id, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?)"
        video_values = [
            (1, "课时1视频", "/video/python/first/01.avi", 1, user_type, course_type, 0),
            (2, "课时2视频", "/video/python/first/8月20日技术分享视频.avi", 2, user_type, course_type, 0),
            (3, "课时3视频", "/video/python/first/02.avi", 3, user_type, course_type, 0),
        ]
        cursor.executemany(insert_video_sql, video_values)
        conn.commit()


class UserService:
    def __init__(self):
        self.is_first_login = True
        self.is_login = False

    def check_first_login(self):
        query_sql = f"SELECT COUNT(*) FROM '{user_table}'"
        cursor.execute(query_sql)
        result = cursor.fetchone()
        existUser = result[0]
        if existUser <= 0:
            self.is_first_login = True
        else:
            self.is_first_login = False
        return self.is_first_login

    def login(self, account, password):
        query_sql = f"SELECT COUNT(*) FROM '{user_table}' WHERE account = '{account}' AND password = '{password}'"
        cursor.execute(query_sql)
        result = cursor.fetchone()
        existUser = result[0]
        if existUser > 0:
            self.is_login = True
            return self.is_login

    def get_one(self):
        query_sql = f"SELECT * FROM '{user_table}'"
        cursor.execute(query_sql)
        return cursor.fetchone()

    def list_user(self):
        query_sql = f"SELECT * FROM '{user_table}'"
        cursor.execute(query_sql)
        return cursor.fetchall()

    def update_user(self, origin_user_id, user_id):
        update_sql = f"UPDATE '{user_table}' SET origin_user_id = ? WHERE id = ?"
        update_values = (origin_user_id, user_id)
        cursor.execute(update_sql, update_values)
        conn.commit()


class UserCourseService:
    def get_one(self):
        query_sql = f"SELECT * FROM '{user_course_table}'"
        cursor.execute(query_sql)
        return cursor.fetchone()

    def list_user_course(self):
        query_sql = f"SELECT * FROM '{user_course_table}'"
        cursor.execute(query_sql)
        return cursor.fetchall()


class HomeworkService:
    def __init__(self):
        self.one = None

    # 作业列表
    def list_homework(self, homework_name=None, section_id=None):
        query_sql = f"SELECT * FROM '{homework_table}'"

        if homework_name is not None:
            query_sql += f" WHERE name LIKE '%{homework_name}%'"
        if section_id is not None:
            if "WHERE" in query_sql:
                query_sql += f" AND section_id = {section_id}"
            else:
                query_sql += f" WHERE section_id = {section_id}"
        print(query_sql)
        cursor.execute(query_sql)
        result = cursor.fetchall()
        print(result)
        return result

    # 作业分数列表
    def list_homework_score(self):
        query_sql = (
            f"SELECT h.id,h.name,h.submit_count,h.display_answer_rule,h.answer_time,us.score,us.submit_count "
            f"FROM '{user_score_table}' AS us "
            f"LEFT JOIN '{homework_table}' h ON h.id = us.source_id "
            f"WHERE us.type = 'HOMEWORK'  "
        )

        print(query_sql)
        cursor.execute(query_sql)
        result = cursor.fetchall()
        print(result)
        return result

    def origin_update_homework(self, homeworks):
        # 数据拼接
        insert_values = []
        for homework in homeworks:
            insert_value = (
                homework["id"],
                homework["name"],
                homework["submit_count"],
                homework["display_answer_rule"],
                homework["answer_time"],
                homework["section_id"],
                homework["description"],
                homework["question_id"],
                homework["user_type"],
                homework["course_type"],
                homework["version"],
            )
            insert_values.append(insert_value)

        # 移除原本地数据
        delete_sql = f"DELETE FROM '{homework_table}'"
        cursor.execute(delete_sql)

        insert_sql = (
            f"INSERT INTO '{homework_table}' (id, name, submit_count, display_answer_rule, "
            f"answer_time, section_id, description, question_id, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"
        )
        cursor.executemany(insert_sql, insert_values)
        conn.commit()


class ExamService:
    def __init__(self):
        self.one = None

    # 考试列表
    def list_exam(self, exam_name=None, course_id=None):
        query_sql = f"SELECT * FROM '{exam_table}'"
        if exam_name is not None:
            query_sql += f" WHERE name LIKE '%{exam_name}%'"
        if course_id is not None:
            if "WHERE" in query_sql:
                query_sql += f" AND course_id = {course_id}"
            else:
                query_sql += f" WHERE course_id = {course_id}"
        print(query_sql)
        cursor.execute(query_sql)
        result = cursor.fetchall()
        print(result)
        return result

    # 考试分数列表
    def list_exam_score(self):
        query_sql = (
            f"SELECT e.id,e.name,e.submit_count,e.display_answer_rule,e.answer_time,us.score,us.submit_count "
            f"FROM '{user_score_table}' AS us "
            f"LEFT JOIN '{exam_table}' e ON e.id = us.source_id "
            f"WHERE us.type = 'EXAM'  "
        )

        print(query_sql)
        cursor.execute(query_sql)
        result = cursor.fetchall()
        print(result)
        return result

    def origin_update_exam(self, exams):
        # 数据拼接
        insert_values = []
        for exam in exams:
            insert_value = (
                exam["id"],
                exam["name"],
                exam["submit_count"],
                exam["display_answer_rule"],
                exam["answer_time"],
                exam["course_id"],
                exam["description"],
                exam["question_id"],
                exam["user_type"],
                exam["course_type"],
                exam["version"],
            )
            insert_values.append(insert_value)

        # 移除原本地数据
        delete_sql = f"DELETE FROM '{exam_table}'"
        cursor.execute(delete_sql)

        insert_sql = (
            f"INSERT INTO '{exam_table}' (id, name, submit_count, display_answer_rule, "
            f"answer_time, course_id, description, question_id, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?,?,?)"
        )
        cursor.executemany(insert_sql, insert_values)
        conn.commit()


class QuestionService:
    def __init__(self):
        self.one = None

    # 题目详情
    def question_detail(self, question_id):
        query_stem_sql = (
            f"SELECT s.id,s.content,s.type,s.score,s.level FROM '{question_table}' AS q "
            f"LEFT JOIN '{question_stem_table}' AS qs ON q.id = qs.question_id "
            f"LEFT JOIN '{stem_table}' AS s ON s.id = qs.stem_id "
            f"WHERE q.id = '{question_id}' "
            f"AND q.enable = 1 AND q.is_delete = 0"
        )

        print(query_stem_sql)
        cursor.execute(query_stem_sql)
        questions = cursor.fetchall()
        print(questions)

        query_answer_sql = (
            f"SELECT a.id,a.content,a.stem_id,a.is_correct FROM '{question_table}' AS q "
            f"LEFT JOIN '{question_stem_table}' AS qs ON q.id = qs.question_id "
            f"LEFT JOIN '{answer_table}' AS a ON a.stem_id = qs.stem_id "
            f"WHERE q.id = '{question_id}' "
            f"AND q.enable = 1 AND q.is_delete = 0"
        )
        print(query_answer_sql)
        cursor.execute(query_answer_sql)
        answers = cursor.fetchall()
        print(answers)

        merged_questions = []
        answers_dict = {}

        # 构建字典，以题目ID为键，对应的答案列表为值
        for answer in answers:
            stem_id = answer[2]
            if stem_id not in answers_dict:
                answers_dict[stem_id] = []
            answers_dict[stem_id].append(answer)

        # 合并题目和答案
        for question in questions:
            stem_id = question[0]
            merged_question = list(question)
            merged_question.insert(4, answers_dict.get(stem_id, []))
            merged_questions.append(tuple(merged_question))

        print(merged_questions)
        return merged_questions

    # 查询提交次数
    def query_submit_count(self, type, source_id):
        """
        type: EXAM  HOMEWORK
        source_id: 作业或考试ID
        """
        user_id = 1
        # 查询提交次数
        query_sql = (
            f"SELECT COUNT(*) FROM '{user_score_table}' WHERE user_id = {user_id} "
            f"AND type = {type} AND source_id = {source_id}  "
        )
        cursor.execute(query_sql)
        result = cursor.fetchone()
        submit_count = result[0]
        return submit_count

    # 提交考试答题
    def save_exam_answers(self, exam_answers):
        """
        1，保存答题记录，
        2，计算此次分数，
        3，保存分数
        """
        print(exam_answers)
        # 考试ID
        exam_id = exam_answers["exam_id"]
        # 用户ID
        user_id = 0
        user_service = UserService()
        user = user_service.get_one()
        if user:
            user_id = user[0]
        else:
            messagebox.showinfo("提示", f"未查询到用户信息")
            return

        # 查询提交次数
        query_sql = (
            f"SELECT COUNT(*) FROM '{user_score_table}' WHERE user_id = {user_id} "
            f"AND type = 'EXAM' AND source_id = {exam_id}  "
        )
        cursor.execute(query_sql)
        result = cursor.fetchone()
        submit_count = result[0] + 1

        print(exam_answers["stems"])
        if exam_answers["stems"] is not None:
            exam_score = 0.0
            exam_answer_log_values = []
            for stem in exam_answers["stems"]:
                print(stem)
                # 答题记录封装
                exam_answer_log_value = (
                    user_id,
                    exam_id,
                    stem["stem_id"],
                    stem["answer_ids"],
                    stem["correct_answer_ids"],
                    0,
                )
                exam_answer_log_values.append(exam_answer_log_value)
                # 2,计算此次分数
                # if stem['stem_type'] == "SINGLE_CHOICE":

                if stem["answer_ids"] == stem["correct_answer_ids"]:
                    exam_score += stem["stem_score"]
                # elif stem['stem_type'] == "MULTIPLE_CHOICE":
                #     print()
                # elif stem['stem_type'] == "JUDGE":
                #     print()
                # elif stem['stem_type'] == "OTHER":
                #     print()
                # else:
                #     print("该类型暂无处理方式")

            print(exam_answer_log_values)
            print(exam_score)
            print(submit_count)
            # 1.1,删除之前的答题记录，
            delete_query = (
                f"DELETE FROM '{exam_answer_log_table}' WHERE user_id = {user_id} "
                f"AND exam_id={exam_id}"
            )
            cursor.execute(delete_query)
            # 1.2,保存这次答题记录
            insert_exam_answer_log_sql = (
                f"INSERT INTO '{exam_answer_log_table}' "
                f"(user_id, exam_id, stem_id, "
                f"answer_ids, correct_answer_ids, version "
                f") VALUES (?, ?, ?, ?, ?, ?)"
            )
            cursor.executemany(insert_exam_answer_log_sql, exam_answer_log_values)
            # 3，保存分数
            user_score_value = (user_id, "EXAM", exam_id, exam_score, submit_count, 0)
            insert_user_score_sql = (
                f"INSERT INTO '{user_score_table}' "
                f"(user_id, 'type', source_id, "
                f"score, submit_count, version "
                f") VALUES (?, ?, ?, ?, ?, ?)"
            )
            cursor.execute(insert_user_score_sql, user_score_value)
            conn.commit()

    # 提交作业答题
    def save_homework_answers(self, homework_answers):
        """
        1，保存答题记录，
        2，计算此次分数，
        3，保存分数
        """
        print(homework_answers)
        # 作业ID
        homework_id = homework_answers["homework_id"]
        # 用户ID
        user_id = 0
        user_service = UserService()
        user = user_service.get_one()
        if user:
            user_id = user[0]
        else:
            messagebox.showinfo("提示", f"未查询到用户信息")
            return
        # 查询提交次数
        query_sql = (
            f"SELECT COUNT(*) FROM '{user_score_table}' WHERE user_id = {user_id} "
            f"AND type = 'HOMEWORK' AND source_id = {homework_id}  "
        )
        cursor.execute(query_sql)
        result = cursor.fetchone()
        submit_count = result[0] + 1

        print(homework_answers["stems"])
        if homework_answers["stems"] is not None:
            homework_score = 0.0
            homework_answer_log_values = []
            for stem in homework_answers["stems"]:
                print(stem)
                # 答题记录封装
                homework_answer_log_value = (
                    user_id,
                    homework_id,
                    stem["stem_id"],
                    stem["answer_ids"],
                    stem["correct_answer_ids"],
                    0,
                )
                homework_answer_log_values.append(homework_answer_log_value)
                # 2,计算此次分数
                if stem["answer_ids"] == stem["correct_answer_ids"]:
                    homework_score += stem["stem_score"]

            print(homework_answer_log_values)
            print(homework_score)
            print(submit_count)
            # 1.1,删除之前的答题记录，
            delete_query = (
                f"DELETE FROM '{homework_answer_log_table}' WHERE user_id = {user_id} "
                f"AND homework_id={homework_id}"
            )
            cursor.execute(delete_query)
            # 1.2,保存这次答题记录
            insert_exam_answer_log_sql = (
                f"INSERT INTO '{homework_answer_log_table}' "
                f"(user_id, homework_id, stem_id, "
                f"answer_ids, correct_answer_ids, version "
                f") VALUES (?, ?, ?, ?, ?, ?)"
            )
            cursor.executemany(insert_exam_answer_log_sql, homework_answer_log_values)
            # 3，保存分数
            user_score_value = (user_id, "HOMEWORK", homework_id, homework_score, submit_count, 0)
            insert_user_score_sql = (
                f"INSERT INTO '{user_score_table}' "
                f"(user_id, 'type', source_id, "
                f"score, submit_count, version "
                f") VALUES (?, ?, ?, ?, ?, ?)"
            )
            cursor.execute(insert_user_score_sql, user_score_value)
            conn.commit()

    def origin_update_question(self, questions):
        # 数据拼接
        insert_values = []
        for question in questions:
            insert_value = (
                question["id"],
                question["title"],
                question["type"],
                question["enable"],
                question["is_delete"],
                question["user_type"],
                question["course_type"],
                question["version"],
            )
            insert_values.append(insert_value)

        # 移除原本地数据
        delete_sql = f"DELETE FROM '{question_table}'"
        cursor.execute(delete_sql)

        insert_sql = f"INSERT INTO '{question_table}' (id, title, type, enable,is_delete, user_type, course_type ,version) VALUES (?, ?, ?, ?, ?, ?,?,?)"
        cursor.executemany(insert_sql, insert_values)
        conn.commit()


class VideoService:
    def list_video(self):
        query_sql = f"SELECT * FROM '{video_table}'"
        cursor.execute(query_sql)
        result = cursor.fetchall()
        return result

    def origin_update_video(self, videos):
        # 数据拼接
        insert_values = []
        for video in videos:
            video_url = video["path"]
            save_path = os.path.join(
                os.path.dirname(__file__), "video", video["course_type"], video["name"]
            )

            # 发送HTTP请求获取视频内容
            response = requests.get(video_url, stream=True)

            # 确保请求成功
            if response.status_code == 200:
                # 以二进制写入方式打开文件，并逐块写入内容
                with open(save_path, "wb") as f:
                    for chunk in response.iter_content(chunk_size=8192):
                        f.write(chunk)
                print("视频下载完成")
            else:
                print("视频下载失败")

            insert_video_value = (
                video["id"],
                video["name"],
                save_path,
                video["section_id"],
                video["user_type"],
                video["course_type"],
                video["version"],
            )
            insert_values.append(insert_video_value)

        # 移除本地视频

        # 移除原本地数据
        delete_sql = f"DELETE FROM '{video_table}'"
        cursor.execute(delete_sql)

        insert_sql = f"INSERT INTO '{video_table}' (id, name, path, section_id, user_type, course_type, version) VALUES (?, ?, ?, ?, ?,?,?)"
        cursor.executemany(insert_sql, insert_values)
        conn.commit()


class StemService:
    def list_stem(self):
        query_sql = f"SELECT * FROM '{stem_table}'"
        cursor.execute(query_sql)
        result = cursor.fetchall()
        return result

    def origin_update_stem(self, stems):
        # 数据拼接
        insert_values = []
        for stem in stems:
            insert_value = (
                stem["id"],
                stem["content"],
                stem["type"],
                float(stem["score"]),
                stem["analysis"],
                stem["level"],
                stem["enable"],
                stem["is_delete"],
                stem["user_type"],
                stem["course_type"],
                stem["version"],
            )
            insert_values.append(insert_value)

        # 移除原本地数据
        delete_sql = f"DELETE FROM '{stem_table}'"
        cursor.execute(delete_sql)

        insert_sql = (
            f"INSERT INTO '{stem_table}' (id, content, `type`, score,analysis, `level`, enable, "
            f"is_delete, user_type, course_type, version) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?,?,?)"
        )
        cursor.executemany(insert_sql, insert_values)
        conn.commit()


class AnswerService:
    def list_answer(self):
        query_sql = f"SELECT * FROM '{stem_table}'"
        cursor.execute(query_sql)
        result = cursor.fetchall()
        return result

    def origin_update_answer(self, answers):
        # 数据拼接
        insert_values = []
        for answer in answers:
            insert_value = (
                answer["id"],
                answer["content"],
                answer["stem_id"],
                answer["is_correct"],
                answer["user_type"],
                answer["course_type"],
                answer["version"],
            )
            insert_values.append(insert_value)

        # 移除原本地数据
        delete_sql = f"DELETE FROM '{answer_table}'"
        cursor.execute(delete_sql)

        insert_sql = f"INSERT INTO '{answer_table}' (id, content, stem_id, is_correct, user_type, course_type,version) VALUES (?, ?, ?, ?, ?,?,?)"
        cursor.executemany(insert_sql, insert_values)
        conn.commit()


class QuestionStemService:
    def list_question_stem(self):
        query_sql = f"SELECT * FROM '{question_stem_table}'"
        cursor.execute(query_sql)
        result = cursor.fetchall()
        return result

    def origin_update_question_stem(self, question_stems):
        # 数据拼接
        insert_values = []
        for question_stem in question_stems:
            insert_value = (
                question_stem["id"],
                question_stem["question_id"],
                question_stem["stem_id"],
                question_stem["user_type"],
                question_stem["course_type"],
                question_stem["version"],
            )
            insert_values.append(insert_value)

        # 移除原本地数据
        delete_sql = f"DELETE FROM '{question_stem_table}'"
        cursor.execute(delete_sql)

        insert_sql = f"INSERT INTO '{question_stem_table}' (id, question_id, stem_id, user_type, course_type,version) VALUES (?, ?, ?, ?,?,?)"
        cursor.executemany(insert_sql, insert_values)
        conn.commit()



from datetime import datetime
import os
import csv
from decimal import Decimal, InvalidOperation
import tkinter as tk
from tkinter import ttk


##################
# Widget Classes #
##################

class ValidatedMixin:
    """Adds a validation functionality to an input widget"""

    def __init__(self, *args, error_var=None, **kwargs):
        self.error = error_var or tk.StringVar()
        super().__init__(*args, **kwargs)

        vcmd = self.register(self._validate)
        invcmd = self.register(self._invalid)
        '''
        @Author ：yb
        @Date   ：2023-08-19 21:50:10
        @Desc   ：
        we need to give it a tuple containing a reference to a Tcl/Tk function, 
        and optionally some substitution codes that specify information
        '''
        self.config(
            validate='all',
            validatecommand=(vcmd, '%P', '%s', '%S', '%V', '%i', '%d'),
            invalidcommand=(invcmd, '%P', '%s', '%S', '%V', '%i', '%d')
        )

    def _toggle_error(self, on=False):
        self.config(foreground=('red' if on else 'black'))

    def _validate(self, proposed, current, char, event, index, action):
        """The validation method.

        Don't override this, override _key_validate, and _focus_validate
        """
        self._toggle_error(False)
        self.error.set('')
        valid = True
        if event == 'focusout':
            valid = self._focusout_validate(event=event)
        elif event == 'key':
            valid = self._key_validate(
                proposed=proposed,
                current=current,
                char=char,
                event=event,
                index=index,
                action=action
            )
        return valid

    def _focusout_validate(self, **kwargs):
        return True

    def _key_validate(self, **kwargs):
        return True

    def _invalid(self, proposed, current, char, event, index, action):
        if event == 'focusout':
            self._focusout_invalid(event=event)
        elif event == 'key':
            self._key_invalid(
                proposed=proposed,
                current=current,
                char=char,
                event=event,
                index=index,
                action=action
            )

    def _focusout_invalid(self, **kwargs):
        """Handle invalid data on a focus event"""
        self._toggle_error(True)

    def _key_invalid(self, **kwargs):
        """Handle invalid data on a key event.

        By default we want to do nothing
        """

        pass

    def trigger_focusout_validation(self):
        valid = self._validate('', '', '', 'focusout', '', '')
        if not valid:
            self._focusout_invalid(event='focusout')
        return valid


class DateEntry(ValidatedMixin, ttk.Entry):

    def _key_validate(self, action, index, char, **kwargs):
        valid = True

        if action == '0':  # This is a delete action
            valid = True
        elif index in ('0', '1', '2', '3', '5', '6', '8', '9'):
            valid = char.isdigit()
        elif index in ('4', '7'):
            valid = char == '-'
        else:
            valid = False
        return valid

    def _focusout_validate(self, event):
        valid = True
        if not self.get():
            self.error.set('A value is required')
            valid = False
        try:
            datetime.strptime(self.get(), '%Y-%m-%d')
        except ValueError:
            self.error.set('Invalid date')
            valid = False
        return valid


class RequiredEntry(ValidatedMixin, ttk.Entry):

    def _focusout_validate(self, event):
        valid = True
        if not self.get():
            valid = False
            self.error.set('A value is required')
        return valid


class ValidatedCombobox(ValidatedMixin, ttk.Combobox):

    def _key_validate(self, proposed, action, **kwargs):
        valid = True
        # if the user tries to delete,
        # just clear the field
        if action == '0':
            self.set('')
            return True

        # get our values list
        values = self.cget('values')
        # Do a case-insensitve match against the entered text
        matching = [
            x for x in values
            if x.lower().startswith(proposed.lower())
        ]
        if len(matching) == 0:
            valid = False
        elif len(matching) == 1:
            self.set(matching[0])
            self.icursor(tk.END)
            valid = False
        return valid

    def _focusout_validate(self, **kwargs):
        valid = True
        if not self.get():
            valid = False
            self.error.set('A value is required')
        return valid


class ValidatedSpinbox(ValidatedMixin, tk.Spinbox):

    def __init__(self, *args, min_var=None, max_var=None,
                 focus_update_var=None, from_='-Infinity', to='Infinity',
                 **kwargs):
        super().__init__(*args, from_=from_, to=to, **kwargs)
        self.resolution = Decimal(str(kwargs.get('increment', '1.0')))
        self.precision = self.resolution.normalize().as_tuple().exponent
        # there should always be a variable,
        # or some of our code will fail
        self.variable = kwargs.get('textvariable') or tk.DoubleVar()

        if min_var:
            self.min_var = min_var
            self.min_var.trace('w', self._set_minimum)
        if max_var:
            self.max_var = max_var
            self.max_var.trace('w', self._set_maximum)
        self.focus_update_var = focus_update_var
        self.bind('<FocusOut>', self._set_focus_update_var)

    def _set_focus_update_var(self, event):
        value = self.get()
        if self.focus_update_var and not self.error.get():
            self.focus_update_var.set(value)

    def _set_minimum(self, *args):
        current = self.get()
        try:
            new_min = self.min_var.get()
            self.config(from_=new_min)
        except (tk.TclError, ValueError):
            pass
        if not current:
            self.delete(0, tk.END)
        else:
            self.variable.set(current)
        self.trigger_focusout_validation()

    def _set_maximum(self, *args):
        current = self.get()
        try:
            new_max = self.max_var.get()
            self.config(to=new_max)
        except (tk.TclError, ValueError):
            pass
        if not current:
            self.delete(0, tk.END)
        else:
            self.variable.set(current)
        self.trigger_focusout_validation()

    def _key_validate(self, char, index, current, proposed, action, **kwargs):
        valid = True
        min_val = self.cget('from')
        max_val = self.cget('to')
        no_negative = min_val >= 0
        no_decimal = self.precision >= 0
        if action == '0':
            return True

        # First, filter out obviously invalid keystrokes
        if any([
                (char not in ('-1234567890.')),
                (char == '-' and (no_negative or index != '0')),
                (char == '.' and (no_decimal or '.' in current))
        ]):
            return False

        # At this point, proposed is either '-', '.', '-.',
        # or a valid Decimal string
        if proposed in '-.':
            return True

        # Proposed is a valid Decimal string
        # convert to Decimal and check more:
        proposed = Decimal(proposed)
        proposed_precision = proposed.as_tuple().exponent

        if any([
            (proposed > max_val),
            (proposed_precision < self.precision)
        ]):
            return False

        return valid

    def _focusout_validate(self, **kwargs):
        valid = True
        value = self.get()
        min_val = self.cget('from')
        max_val = self.cget('to')

        try:
            value = Decimal(value)
        except InvalidOperation:
            self.error.set('Invalid number string: {}'.format(value))
            return False

        if value < min_val:
            self.error.set('Value is too low (min {})'.format(min_val))
            valid = False
        if value > max_val:
            self.error.set('Value is too high (max {})'.format(max_val))

        return valid

##################
# Module Classes #
##################


class LabelInput(tk.Frame):
    """A widget containing a label and input together."""

    def __init__(self, parent, label='', input_class=ttk.Entry,
                 input_var=None, input_args=None, label_args=None,
                 **kwargs):
        super().__init__(parent, **kwargs)
        input_args = input_args or {}
        label_args = label_args or {}
        self.variable = input_var

        if input_class in (ttk.Checkbutton, ttk.Button, ttk.Radiobutton):
            input_args["text"] = label
            input_args["variable"] = input_var
        else:
            self.label = ttk.Label(self, text=label, **label_args)
            self.label.grid(row=0, column=0, sticky=(tk.W + tk.E))
            input_args["textvariable"] = input_var

        self.input = input_class(self, **input_args)
        self.input.grid(row=1, column=0, sticky=(tk.W + tk.E))
        self.columnconfigure(0, weight=1)
        self.error = getattr(self.input, 'error', tk.StringVar())
        self.error_label = ttk.Label(self, textvariable=self.error)
        self.error_label.grid(row=2, column=0, sticky=(tk.W + tk.E))

    def grid(self, sticky=(tk.E + tk.W), **kwargs):
        super().grid(sticky=sticky, **kwargs)

    def get(self):
        try:
            if self.variable:
                return self.variable.get()
            elif type(self.input) == tk.Text:
                return self.input.get('1.0', tk.END)
            else:
                return self.input.get()
        except (TypeError, tk.TclError):
            # happens when numeric fields are empty.
            return ''

    def set(self, value, *args, **kwargs):
        if type(self.variable) == tk.BooleanVar:
                self.variable.set(bool(value))
        elif self.variable:
                self.variable.set(value, *args, **kwargs)
        elif type(self.input) in (ttk.Checkbutton, ttk.Radiobutton):
            if value:
                self.input.select()
            else:
                self.input.deselect()
        elif type(self.input) == tk.Text:
            self.input.delete('1.0', tk.END)
            self.input.insert('1.0', value)
        else:
            self.input.delete(0, tk.END)
            self.input.insert(0, value)


class DataRecordForm(tk.Frame):
    """The input form for our widgets"""

    def __init__(self, parent, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)
        # A dict to keep track of input widgets
        self.inputs = {}

        # Build the form
        # recordinfo section
        recordinfo = tk.LabelFrame(self, text="Record Information")

         # line 1
        self.inputs['Date'] = LabelInput(
            recordinfo, "Date",
            input_class=DateEntry,
            input_var=tk.StringVar()
        )
        self.inputs['Date'].grid(row=0, column=0)
        self.inputs['Time'] = LabelInput(
            recordinfo, "Time",
            input_class=ValidatedCombobox,
            input_var=tk.StringVar(),
            input_args={"values": ["8:00", "12:00", "16:00", "20:00"]}
        )
        self.inputs['Time'].grid(row=0, column=1)
        self.inputs['Technician'] = LabelInput(
            recordinfo, "Technician",
            input_class=RequiredEntry,
            input_var=tk.StringVar()
        )
        self.inputs['Technician'].grid(row=0, column=2)

       # line 2
        self.inputs['Lab'] = LabelInput(
            recordinfo, "Lab",
            input_class=ValidatedCombobox,
            input_var=tk.StringVar(),
            input_args={"values": ["A", "B", "C", "D", "E"]}
        )
        self.inputs['Lab'].grid(row=1, column=0)
        self.inputs['Plot'] = LabelInput(
            recordinfo, "Plot",
            input_class=ValidatedCombobox,
            input_var=tk.StringVar(),
            input_args={"values": [str(x) for x in range(1, 21)]}
        )
        self.inputs['Plot'].grid(row=1, column=1)
        self.inputs['Seed sample'] = LabelInput(
            recordinfo, "Seed sample",
            input_class=RequiredEntry,
            input_var=tk.StringVar()
        )
        self.inputs['Seed sample'].grid(row=1, column=2)

        recordinfo.grid(row=0, column=0, sticky="we")

        # Environment Data
        environmentinfo = tk.LabelFrame(self, text="Environment Data")
        self.inputs['Humidity'] = LabelInput(
            environmentinfo, "Humidity (g/m³)",
            input_class=ValidatedSpinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": '0.5', "to": '52.0', "increment": '.01'}
        )
        self.inputs['Humidity'].grid(row=0, column=0)
        self.inputs['Light'] = LabelInput(
            environmentinfo, "Light (klx)",
            input_class=ValidatedSpinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": '0', "to": '100', "increment": '.01'}
        )
        self.inputs['Light'].grid(row=0, column=1)
        self.inputs['Temperature'] = LabelInput(
            environmentinfo, "Temperature (°C)",
            input_class=ValidatedSpinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": '4', "to": '40', "increment": '.01'}
        )
        self.inputs['Temperature'].grid(row=0, column=2)
        self.inputs['Equipment Fault'] = LabelInput(
            environmentinfo, "Equipment Fault",
            input_class=ttk.Checkbutton,
            input_var=tk.BooleanVar()
        )
        self.inputs['Equipment Fault'].grid(row=1, column=0, columnspan=3)
        environmentinfo.grid(row=1, column=0, sticky="we")

        # Plant Data section
        plantinfo = tk.LabelFrame(self, text="Plant Data")

        self.inputs['Plants'] = LabelInput(
            plantinfo, "Plants",
            input_class=ValidatedSpinbox,
            input_var=tk.IntVar(),
            input_args={"from_": '0', "to": '20'}
        )
        self.inputs['Plants'].grid(row=0, column=0)
        self.inputs['Blossoms'] = LabelInput(
            plantinfo, "Blossoms",
            input_class=ValidatedSpinbox,
            input_var=tk.IntVar(),
            input_args={"from_": '0', "to": '1000'}
        )
        self.inputs['Blossoms'].grid(row=0, column=1)
        self.inputs['Fruit'] = LabelInput(
            plantinfo, "Fruit",
            input_class=ValidatedSpinbox,
            input_var=tk.IntVar(),
            input_args={"from_": '0', "to": '1000'}
        )
        self.inputs['Fruit'].grid(row=0, column=2)
        # Height data
        # create variables to be updated for min/max height
        # they can be referenced for min/max variables
        min_height_var = tk.DoubleVar(value='-infinity')
        max_height_var = tk.DoubleVar(value='infinity')

        self.inputs['Min Height'] = LabelInput(
            plantinfo, "Min Height (cm)",
            input_class=ValidatedSpinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": '0', "to": '1000', "increment": '.01',
                        "max_var": max_height_var,
                        "focus_update_var": min_height_var}
        )
        self.inputs['Min Height'].grid(row=1, column=0)
        self.inputs['Max Height'] = LabelInput(
            plantinfo, "Max Height (cm)",
            input_class=ValidatedSpinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": '0', "to": '1000', "increment": '.01',
                        "min_var": min_height_var,
                        "focus_update_var": max_height_var}
        )
        self.inputs['Max Height'].grid(row=1, column=1)
        self.inputs['Median Height'] = LabelInput(
            plantinfo, "Median Height (cm)",
            input_class=ValidatedSpinbox,
            input_var=tk.DoubleVar(),
            input_args={"from_": '0', "to": '1000', "increment": '.01',
                        "min_var": min_height_var,
                        "max_var": max_height_var}
        )
        self.inputs['Median Height'].grid(row=1, column=2)

        plantinfo.grid(row=2, column=0, sticky="we")

        # Notes section
        self.inputs['Notes'] = LabelInput(
            self, "Notes",
            input_class=tk.Text,
            input_args={"width": 75, "height": 10}
        )
        self.inputs['Notes'].grid(sticky="w", row=3, column=0)

        # default the form
        self.reset()

    def get(self):
        """Retrieve data from form as a dict"""

        # We need to retrieve the data from Tkinter variables
        # and place it in regular Python objects

        data = {}
        for key, widget in self.inputs.items():
            data[key] = widget.get()
        return data

    def reset(self):
        """Resets the form entries"""

        # gather the values to keep for each lab
        lab = self.inputs['Lab'].get()
        time = self.inputs['Time'].get()
        technician = self.inputs['Technician'].get()
        plot = self.inputs['Plot'].get()
        plot_values = self.inputs['Plot'].input.cget('values')

        # clear all values
        for widget in self.inputs.values():
            widget.set('')

        current_date = datetime.today().strftime('%Y-%m-%d')
        self.inputs['Date'].set(current_date)
        self.inputs['Time'].input.focus()

        # check if we need to put our values back, then do it.
        if plot not in ('', plot_values[-1]):
            self.inputs['Lab'].set(lab)
            self.inputs['Time'].set(time)
            self.inputs['Technician'].set(technician)
            next_plot_index = plot_values.index(plot) + 1
            self.inputs['Plot'].set(plot_values[next_plot_index])
            self.inputs['Seed sample'].input.focus()


    def get_errors(self):
        """Get a list of field errors in the form"""

        errors = {}
        for key, widget in self.inputs.items():
            if hasattr(widget.input, 'trigger_focusout_validation'):
                widget.input.trigger_focusout_validation()
            if widget.error.get():
                errors[key] = widget.error.get()

        return errors


class Application(tk.Tk):
    """Application root window"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.title("ABQ Data Entry Application")
        self.resizable(width=False, height=False)

        ttk.Label(
            self,
            text="ABQ Data Entry Application",
            font=("TkDefaultFont", 16)
        ).grid(row=0)


        self.recordform = DataRecordForm(self)
        self.recordform.grid(row=1, padx=10)

        self.savebutton = ttk.Button(self, text="Save", command=self.on_save)
        self.savebutton.grid(sticky="e", row=2, padx=10)

        # status bar
        self.status = tk.StringVar()
        self.statusbar = ttk.Label(self, textvariable=self.status)
        self.statusbar.grid(sticky="we", row=3, padx=10)

        self.records_saved = 0

    def on_save(self):
        """Handles save button clicks"""

        # Check for errors first

        errors = self.recordform.get_errors()
        if errors:
            self.status.set(
                "Cannot save, error in fields: {}"
                .format(', '.join(errors.keys()))
            )
            return False

        # For now, we save to a hardcoded filename with a datestring.
        # If it doesnt' exist, create it,
        # otherwise just append to the existing file
        datestring = datetime.today().strftime("%Y-%m-%d")
        filename = "abq_data_record_{}.csv".format(datestring)
        newfile = not os.path.exists(filename)

        data = self.recordform.get()

        with open(filename, 'a') as fh:
            csvwriter = csv.DictWriter(fh, fieldnames=data.keys())
            if newfile:
                csvwriter.writeheader()
            csvwriter.writerow(data)

        self.records_saved += 1
        self.status.set(
            "{} records saved this session".format(self.records_saved)
        )
        self.recordform.reset()


if __name__ == "__main__":

    app = Application()
    app.mainloop()



import tkinter as tk
from datetime import datetime
from tkinter import ttk

from tkcalendar import DateEntry


class DateTimePicker:
    def __init__(self, parent, show_text):
        self.parent = parent
        self.selected_datetime = None

        # 创建一个标签显示 "开始时间"
        label_start_time = ttk.Label(self.parent, text=show_text)
        label_start_time.pack()

        # 创建日历组件
        self.calendar = DateEntry(
            self.parent, width=12, background="darkblue", foreground="white", borderwidth=2
        )
        self.calendar.pack(pady=10)

        # 创建时间选择组件
        self.time_frame = ttk.Frame(self.parent)

        # 创建小时选择下拉菜单
        hour_label = ttk.Label(self.time_frame, text="小时：")
        hour_label.grid(row=0, column=0, padx=5, pady=5)
        self.hour_combo = ttk.Combobox(self.time_frame, values=[str(i).zfill(2) for i in range(24)])
        self.hour_combo.current(0)
        self.hour_combo.grid(row=0, column=1, padx=5, pady=5)

        # 创建分钟选择下拉菜单
        minute_label = ttk.Label(self.time_frame, text="分钟：")
        minute_label.grid(row=1, column=0, padx=5, pady=5)
        self.minute_combo = ttk.Combobox(
            self.time_frame, values=[str(i).zfill(2) for i in range(60)]
        )
        self.minute_combo.current(0)
        self.minute_combo.grid(row=1, column=1, padx=5, pady=5)

        self.time_frame.pack()

        # # 创建按钮用于获取选定的日期和时间
        # get_time_button = ttk.Button(self.parent, text="获取时间", command=self.get_selected_datetime)
        # get_time_button.pack(pady=10)

    def get_selected_datetime(self):
        date = self.calendar.get_date()
        date_str = date.strftime("%m/%d/%y")  # 将日期对象转换为字符串
        time = f"{self.hour_combo.get()}:{self.minute_combo.get()}:00"

        # 将日期时间转换为指定格式
        self.selected_datetime = (
            datetime.strptime(date_str, "%m/%d/%y").strftime("%Y-%m-%d") + " " + time
        )
        return self.selected_datetime
